﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Sandbox.Plugins;

namespace Sandbox.Controls
{
    public partial class IdentSwapDialog : Form
    {
        private int ident;

        public IdentSwapDialog() 
        {
            InitializeComponent();

            comboBox1.Text = "null";
            comboBox2.Text = "null";
            
            //Load list of tag classes
            Load_Tag_Class_List();
        }

        public IdentSwapDialog(string tagname)
        {
            InitializeComponent();

            Ident = tagname;

            if (ident!= -1)
            {                
                foreach (MapPlugin.Tag t in GlobalVariables.Plugin.Tags)
                {
                    if (t.Ident == ident)
                    {
                        comboBox1.Text = t.Type;
                        comboBox2.Text = t.Filename;
                    }
                }
                Load_Tag_List(comboBox1.Text);
            }
            else
            {
                //If Ident is NULL set control values to NULL
                comboBox1.Text = "null";
                comboBox2.Text = "null";
            }
            //Load list of tag classes
            Load_Tag_Class_List();  
        }

        public string SwapButtonText
        {
            get
            {
                return this.btnSwap.Text;
            }
            set
            {
                this.btnSwap.Text = value;
            }
        }

        public string Ident
        {
            get { return GlobalVariables.Plugin.GetTagNameFromID(ident); }
            set { ident = GlobalVariables.Plugin.GetIDFromTagName(value); }
        }
        /// <summary>
        /// Load tag list a tag names for selected tag type
        /// </summary>
        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            Load_Tag_List(comboBox1.Items[comboBox1.SelectedIndex].ToString());
        }
        /// <summary>
        /// Loads a list of Tag classes into the control
        /// </summary>
        public void Load_Tag_Class_List()
        {
            comboBox1.Items.Clear();
            for (int i = 0; i < GlobalVariables.Plugin.TagClasses.Count; i++)
            {
                comboBox1.Items.Add(GlobalVariables.Plugin.TagClasses[i]);
            }
        }
        /// <summary>
        /// Loads a list of Tag names into the control
        /// </summary>
        /// <param name="type"></param>
        public void Load_Tag_List(string type)
        {
            comboBox2.Items.Clear();
            for (int i = 0; i < GlobalVariables.Plugin.Tags.Count; i++)
            {
                if (type == GlobalVariables.Plugin.Tags[i].Type)
                {
                    comboBox2.Items.Add(GlobalVariables.Plugin.Tags[i].Filename);
                }
            }
        }
        private void btnSwap_Click(object sender, EventArgs e)
        {
            Ident = comboBox2.Text;
            this.DialogResult = DialogResult.OK;
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
        }
    }
}
