﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Sandbox.Editor;
using Sandbox.Usermap;
using System.Reflection;
using Sandbox.Plugins;

namespace Sandbox.Controls
{
    /// <summary>
    /// Displays and allows editing of usermap data
    /// </summary>
    public partial class ListpanelV2 : UserControl
    {
        /// <summary>
        /// Creates new ListpanelV2 instance.
        /// </summary>
        public ListpanelV2()
        {
            //Initialize form components
            InitializeComponent();            
        }
        /// <summary>
        /// Load listview with usermaps data
        /// </summary>
        public void Populate()
        {
            //Clear anything that might be in the listview
            listView1.Items.Clear();
            listView1.LargeImageList = GlobalVariables.ImageIcons; //Set large icon list
            listView1.SmallImageList = GlobalVariables.SmallImageIcons; //Set small icon list
            if (placementBlocksToolStripMenuItem.Checked)
            {
                //For each placement block
                for (int i = 0; i < 640; i++)
                {
                    ListViewItem li = new ListViewItem(); //Create new listview item
                    int tIndex = GlobalVariables.Usermap.placementBlocks[i].TagIndex; //Get current blocks tag index
                    if (tIndex != -1) //If tag index is not null
                    {
                        li.Text = GlobalVariables.Plugin.GetTagNameFromID(GlobalVariables.Usermap.tagIndex[tIndex].ident); //Set list view items name to tag path
                        li.ImageKey = li.Text.Replace('\\', '_'); //Set image key               
                    }
                    else //If tag index is null
                    {
                        li.Text = "Empty"; //Lable listview item as "Empty"
                    }
                    //Set list view items to proper listview group
                    if (blockTypeToolStripMenuItem1.Checked)
                    {
                        switch (GlobalVariables.Usermap.placementBlocks[i].BlockType)
                        {
                            case H3Usermap.PlacementBlock.blockType.Reserved:
                            case H3Usermap.PlacementBlock.blockType.Original:
                            case H3Usermap.PlacementBlock.blockType.Added:
                            case H3Usermap.PlacementBlock.blockType.Edited:
                            case H3Usermap.PlacementBlock.blockType.Player_Spawn:
                            case H3Usermap.PlacementBlock.blockType.NULL:
                                li.Group = listView1.Groups[GlobalVariables.Usermap.placementBlocks[i].BlockType.ToString()];
                                break;
                            //In the rare case a placement block 
                            //does not fall into one of the above catagories
                            default:
                                li.Group = listView1.Groups["Unsorted"];
                                break;
                        }
                    }
                    else
                    {
                        if (tIndex != -1) //If tag index is not null
                            li.Group = listView1.Groups[GlobalVariables.Plugin.GetTagTypeFromID(
                                GlobalVariables.Usermap.tagIndex[tIndex].ident)];
                        else
                            li.Group = listView1.Groups["no class"];
                    }

                    li.SubItems.Add(i.ToString()); //Add placement blocks index as a subitem
                    li.Tag = i; //Set to tag index to we can get it later
                    if (txtSearch.Text.Equals("")) //If no search string was entered
                    {
                        if (CheckIfBlockIsShown(GlobalVariables.Usermap.placementBlocks[i].BlockType))
                            listView1.Items.Add(li); //Add item to list view
                    }
                    else //If a search string was entered
                    {
                        if (li.Text.Contains(txtSearch.Text)) //Check is items tag path contains the string
                        {
                            if (CheckIfBlockIsShown(GlobalVariables.Usermap.placementBlocks[i].BlockType))
                                listView1.Items.Add(li); //If it does, add the items to list view  
                        }
                    }
                }
            }
            else
            {
                //For each tag index entry
                for (int i = 0; i < 255; i++)
                {
                    ListViewItem li = new ListViewItem(); //Create new listview item
                    int ident = GlobalVariables.Usermap.tagIndex[i].ident;
                    if (ident != -1)
                    {
                        li.Text = GlobalVariables.Plugin.GetTagNameFromID(ident); //Set list view items name to tag path
                        li.ImageKey = li.Text.Replace('\\', '_'); //Set image key 
                    }
                    else
                    {
                        li.Text = "Null Entry";
                    }
                    li.SubItems.Add(i.ToString()); //Add tag entrys index as a subitem
                    li.Tag = i; //Set to index to we can get it later
                    if (txtSearch.Text.Equals("")) //If no search string was entered
                    {
                        listView1.Items.Add(li); //Add item to list view
                    }
                    else //If a search string was entered
                    {
                        if (li.Text.Contains(txtSearch.Text)) //Check is items tag path contains the string
                        {
                            listView1.Items.Add(li); //If it does, add the items to list view  
                        }
                    }
                }
            }
        }
        /// <summary>
        /// Clears the listview
        /// </summary>
        public void Clear()
        {
            //Clear listview items
            listView1.Items.Clear();
        }
        /// <summary>
        /// Checks is a group type is set to be displayed
        ///  in the listview or not.
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public bool CheckIfBlockIsShown(H3Usermap.PlacementBlock.blockType type)
        {
            switch (type)
            {
                case H3Usermap.PlacementBlock.blockType.Reserved:
                    return reservedSlotsToolStripMenuItem.Checked;
                case H3Usermap.PlacementBlock.blockType.Original:
                    return originalSlotsToolStripMenuItem.Checked;
                case H3Usermap.PlacementBlock.blockType.Added:
                    return addedSlotsToolStripMenuItem.Checked;
                case H3Usermap.PlacementBlock.blockType.Edited:
                    return editedSlotsToolStripMenuItem.Checked;
                case H3Usermap.PlacementBlock.blockType.Player_Spawn:
                    return playerSpawnsToolStripMenuItem.Checked;
                case H3Usermap.PlacementBlock.blockType.NULL:
                    return emptySlotsToolStripMenuItem.Checked;
                //In the rare case a placement block 
                //does not fall into one of the above catagories
                default:
                    return unsortedSlotsToolStripMenuItem.Checked;
            }
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            //Determine view mode selected
            switch (comboBox1.Text)
            {
                case "Large Icons":
                    listView1.View = View.LargeIcon; //View = Large Icon
                    break;
                case "Small Icons":
                    listView1.View = View.SmallIcon; //View = Small Icon
                    break;
                case "List":
                    listView1.View = View.List; //View = List
                    break;
                case "Details":
                    listView1.View = View.Details; //View = Details
                    break;
                case "Tile":
                    listView1.View = View.Tile; //View = Tile
                    break;
            }
        }

        private void btnCopy_Click(object sender, EventArgs e)
        {
            //If a usermap is open
            if (GlobalVariables.Usermap != null)
            {
                if(placementBlocksToolStripMenuItem.Checked)
                    //Copy currently selected placement block to our clipboard
                    GlobalVariables.Clipboard = GlobalVariables.Usermap.placementBlocks[(int)listView1.SelectedItems[0].Tag];
                else
                    GlobalVariables.Clipboard = GlobalVariables.Usermap.tagIndex[(int)listView1.SelectedItems[0].Tag];
                    
            }
        }

        private void btnPaste_Click(object sender, EventArgs e)
        {
            //If a usermap is open
            if (GlobalVariables.Usermap != null && GlobalVariables.Clipboard != null)
            {
                if (placementBlocksToolStripMenuItem.Checked)
                    //Copy the clipboard to the currently selected placement block
                    GlobalVariables.Usermap.placementBlocks[(int)listView1.SelectedItems[0].Tag] = (H3Usermap.PlacementBlock)GlobalVariables.Clipboard;
                else
                    GlobalVariables.Usermap.tagIndex[(int)listView1.SelectedItems[0].Tag] = (H3Usermap.TagIndexEntry)GlobalVariables.Clipboard;
                pe_FormClosing(null, null);
            }
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            //If a usermap is open
            if (GlobalVariables.Usermap != null)
            {
                //Create new instance of our property editor
                // and pass it the currently selected object.
                PropertyEditor pe;
                if (placementBlocksToolStripMenuItem.Checked)
                    pe = new PropertyEditor(GlobalVariables.Usermap.placementBlocks[(int)listView1.SelectedItems[0].Tag]);
                else
                    pe = new PropertyEditor(GlobalVariables.Usermap.tagIndex[(int)listView1.SelectedItems[0].Tag]);
                pe.FormClosing += new FormClosingEventHandler(pe_FormClosing);
                pe.ShowDialog(); //Show property editor
            }
        }

        private void btnSwap_Click(object sender, EventArgs e)
        {
            //If a usermap is open
            if (GlobalVariables.Usermap != null)
            {
                //Get the tag index of the currently selected placement block
                int index = GlobalVariables.Usermap.placementBlocks[(int)listView1.SelectedItems[0].Tag].TagIndex;
                if (index != -1) //If placement block is not null
                {
                    //Create new instance of our property editor
                    // and pass it the currently selected placement blocks
                    //  tag index.
                    PropertyEditor pe = new PropertyEditor(GlobalVariables.Usermap.tagIndex[index]);
                    //Set our custom form closing event.
                    pe.FormClosing += new FormClosingEventHandler(pe_FormClosing);
                    pe.ShowDialog(); //Show property editor 
                }
                else //If placement block is null
                {

                }
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            //If a usermap is open
            if (GlobalVariables.Usermap != null)
            {
                if (placementBlocksToolStripMenuItem.Checked)
                {
                    //Create a new placement block
                    H3Usermap.PlacementBlock nullBlock = new H3Usermap.PlacementBlock();
                    nullBlock.LoadNull(); //Load with null values
                    //Set the currently selected placement block
                    // to the null block we created.
                    GlobalVariables.Usermap.placementBlocks[(int)listView1.SelectedItems[0].Tag] = nullBlock;
                    //Update listview item to show as empty
                    listView1.SelectedItems[0].Text = "Empty";
                    listView1.SelectedItems[0].ImageKey = "Null";
                }
                else
                {
                    H3Usermap.TagIndexEntry nullEntry = new H3Usermap.TagIndexEntry();
                    nullEntry.LoadNull();
                    GlobalVariables.Usermap.tagIndex[(int)listView1.SelectedItems[0].Tag] = nullEntry;
                    listView1.SelectedItems[0].Text = "Null Entry";
                    listView1.SelectedItems[0].ImageKey = "";
                }
            }
        }
        /// <summary>
        /// Updates a listview item after a property 
        ///  editor is closed.
        /// </summary>
        void pe_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (placementBlocksToolStripMenuItem.Checked)
            {
                //Get the tag index of the currently selected placement block
                int tIndex = GlobalVariables.Usermap.placementBlocks[(int)listView1.SelectedItems[0].Tag].TagIndex;
                if (tIndex != -1) //If index is not null
                {
                    //Update listview item
                    listView1.SelectedItems[0].Text = GlobalVariables.Plugin.GetTagNameFromID(GlobalVariables.Usermap.tagIndex[tIndex].ident);
                    listView1.SelectedItems[0].ImageKey = listView1.SelectedItems[0].Text.Replace('\\', '_');
                }
                else //If index is null
                {
                    //Update listview item
                    listView1.SelectedItems[0].Text = "Empty";
                    listView1.SelectedItems[0].ImageKey = "Empty";
                }
                listView1.SelectedItems[0].Group = listView1.Groups[GlobalVariables.Usermap.placementBlocks[(int)listView1.SelectedItems[0].Tag].BlockType.ToString()];
                listView1.SelectedItems[0].EnsureVisible();
            }
            else
            {
                int ident = GlobalVariables.Usermap.tagIndex[(int)listView1.SelectedItems[0].Tag].ident;
                if (ident != -1)
                {
                    listView1.SelectedItems[0].Text = GlobalVariables.Plugin.GetTagNameFromID(ident);
                    listView1.SelectedItems[0].ImageKey = listView1.SelectedItems[0].Text.Replace('\\', '_');
                }
                else
                {
                    listView1.SelectedItems[0].Text = "Null Entry";
                    listView1.SelectedItems[0].ImageKey = "";
                }
            }
        }

        private void txtSearch_TextChanged(object sender, EventArgs e)
        {
            //If a usermap is open
            if (GlobalVariables.Usermap != null)
            {
                if (nameToolStripMenuItem.Checked == true)
                {
                    //Hide listview to speed up repopulation
                    listView1.Hide();
                    //Repopulate the listview.
                    Populate();
                    //Show the listview
                    listView1.Show();
                }
                else
                {
                    int sNum = 0;
                    if(int.TryParse(txtSearch.Text, out sNum))
                    {
                        if(sNum >=0 && sNum <=639)
                            listView1.Items[sNum].EnsureVisible();
                    }
                }
            }
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            //If a usermap is open
            if (GlobalVariables.Usermap != null)
            {
                //Recalculate spawned object count
                byte objCount = (byte)GlobalVariables.Usermap.baseObjectCount;
                for (int i = 0; i < 640; i++)
                {
                    if (GlobalVariables.Usermap.placementBlocks[i].BlockType != H3Usermap.PlacementBlock.blockType.NULL)
                        objCount++;
                }
                //Set new object count
                GlobalVariables.Usermap.unamedBlock1.Spawned_Object_Count = objCount;
                //Write changes
                int error = GlobalVariables.Usermap.Write();
                //Show error message
                if (error == 0)
                {
                    SuccessMessage sm = new SuccessMessage("Changes saved.");
                    sm.ShowDialog();
                }
                else
                {
                    MessageBox.Show("Changes were not saved.", "Error");
                }
            }
        }

        private void listView1_MouseDown(object sender, MouseEventArgs e)
        {
            //If right mouse button was clicked
            if (e.Button == MouseButtons.Right)
            {
                //Test where the mouse was clicked
                ListViewHitTestInfo hitTestInfo = listView1.HitTest(e.X, e.Y);
                //If clicked area was not empty
                if (hitTestInfo.Item != null)
                {
                    //show the context menu strip
                    cmsOptions.Show(this, e.X, e.Y + 30);
                }
            }
        }

        private void propertiesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Create new info form and display
            // info of currently selected placement block.
            Info.Show((int)listView1.SelectedItems[0].Tag);
        }

        private void nameToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (GlobalVariables.Usermap != null)
            {
                indexToolStripMenuItem.Checked = !nameToolStripMenuItem.Checked;
                txtSearch.Text = "";
                listView1.Hide();
                Populate();
                listView1.Show();
            }
            else
                nameToolStripMenuItem.Checked = true;
        }

        private void indexToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (GlobalVariables.Usermap != null)
            {
                nameToolStripMenuItem.Checked = !indexToolStripMenuItem.Checked;
                txtSearch.Text = "";
                listView1.Hide();
                Populate();
                listView1.Show();
            }
            else
                indexToolStripMenuItem.Checked = false;
        }

        public void GroupItems()
        {
            if (blockTypeToolStripMenuItem1.Checked)
            {
                listView1.ShowGroups = true;
                listView1.Groups.Clear();
                listView1.Groups.Add("Reserved", "Reserved Slots");
                listView1.Groups.Add("Original", "Original Slots");
                listView1.Groups.Add("Added", "Added Slots");
                listView1.Groups.Add("Edited", "Edited Slots");
                listView1.Groups.Add("Unsorted", "Unsorted Slots");
                listView1.Groups.Add("Player_Spawn", "Player Spawns");
                listView1.Groups.Add("NULL", "Empty Slots");
                Populate();
            }
            else if (tagTypeToolStripMenuItem1.Checked)
            {
                listView1.ShowGroups = true;
                listView1.Groups.Clear();
                foreach (string s in GlobalVariables.Plugin.TagClasses)
                    listView1.Groups.Add(s, s);
                listView1.Groups.Add("no class", "no class");
                Populate();
            }
            else if (noGroupingToolStripMenuItem.Checked)
            {
                listView1.ShowGroups = false;
                Populate();
            }
        }

        private void blockTypeToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (GlobalVariables.Usermap != null)
            {
                noGroupingToolStripMenuItem.Checked = false;
                tagTypeToolStripMenuItem1.Checked = false;
                blockTypeToolStripMenuItem1.Checked = true;
                GroupItems();
            }
            else
                blockTypeToolStripMenuItem1.Checked = true;
        }

        private void tagTypeToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (GlobalVariables.Usermap != null)
            {
                noGroupingToolStripMenuItem.Checked = false;
                blockTypeToolStripMenuItem1.Checked = false;
                tagTypeToolStripMenuItem1.Checked = true;
                GroupItems();
            }
            else
                tagTypeToolStripMenuItem1.Checked = false;
        }

        private void noGroupingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (GlobalVariables.Usermap != null)
            {
                blockTypeToolStripMenuItem1.Checked = false;
                tagTypeToolStripMenuItem1.Checked = false;
                noGroupingToolStripMenuItem.Checked = true;
                GroupItems();
            }
            else
                noGroupingToolStripMenuItem.Checked = false;
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (GlobalVariables.Usermap != null)
                Populate();
        }

        private void placementBlocksToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (GlobalVariables.Usermap != null)
            {
                tagIndexToolStripMenuItem.Checked = false;
                placementBlocksToolStripMenuItem.Checked = true;
                listView1.ShowGroups = !noGroupingToolStripMenuItem.Checked;
                viewByToolStripMenuItem.Enabled = true;
                groupByToolStripMenuItem1.Enabled = true;
                Populate();
            }
        }

        private void tagIndexToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (GlobalVariables.Usermap != null)
            {
                placementBlocksToolStripMenuItem.Checked = false;
                tagIndexToolStripMenuItem.Checked = true;
                listView1.ShowGroups = false;
                viewByToolStripMenuItem.Enabled = false;
                groupByToolStripMenuItem1.Enabled = false;
                Populate();
            }
        }

        private void mapInformationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PropertyEditor pe = new PropertyEditor();
            pe.SelectedObject = GlobalVariables.Usermap.header;
            pe.ShowDialog();
        }
    }
}
