using System;
using System.Collections.Generic;
using System.Text;
using Sandbox.IO;

namespace Sandbox.Data
{
    public class Bitmask8 : Bitmask
    {
        #region Layout
        public byte value;
        #endregion

        public Bitmask8() { }

        public Bitmask8(EndianReader br, string[] option_names)
        {
            base.Read(br);

            Options = option_names;

            GetFlags();
        }
    }

    public class Bitmask16 : Bitmask
    {
        #region Layout
        public short value;
        #endregion

        public Bitmask16(EndianReader br, string[] option_names)
        {
            base.Read(br);

            Options = option_names;

            GetFlags();
        }
    }

    public class Bitmask32 : Bitmask
    {
        #region Layout
        public int value;
        #endregion

        public Bitmask32(EndianReader br, string[] option_names)
        {
            base.Read(br);

            Options = option_names;

            GetFlags();
        }
    }

    public class Bitmask : Chunk
    {
        private string[] options;
        private bool[] flags;

        public override void Read(EndianReader br)
        {
            base.Read(br);
        }

        public override void Write(EndianWriter bw)
        {
            base.Write(bw);
        }

        public string[] Options { get { return options; } set { options = value; } }

        public bool[] Flags { get { return flags; } set { flags = value; } }

        public void GetFlags()
        {
            if (this.GetType().GetField("value").GetValue(this) == typeof(int))
                flags = new bool[32];
            else if (this.GetType().GetField("value").GetValue(this) == typeof(short))
                flags = new bool[16];
            else if (this.GetType().GetField("value").GetValue(this).GetType() == typeof(byte))
                flags = new bool[8];

            uint bit = 1;
            uint temp = Convert.ToUInt32(this.GetType().GetField("value").GetValue(this));
            for (int i = 0; i < flags.Length; i++)
            {
                flags[i] = (bit & temp) == bit ? true : false;
                bit <<= 1;
            }
        }
    }

    /*public class StringID : Chunk
    {
        #region Layout
        public int Index;
        #endregion

        public StringID(EndianReader br, Map.Map Map)
        {
            Read(br);
        }

        public override void Read(EndianReader br)
        {
            base.Read(br);
        }
    }*/
    /// <summary>
    /// Provides functions to Read and Write Halo 3 Tag References
    /// </summary>
    public class TagReference : Chunk
    {
        //Specifies the layout of a Halo3 TagReference we can read it
        #region Layout
        /// <summary>
        /// Specifies type of tag being referenced
        /// </summary>
        public string TagClass;
        /// <summary>
        /// Unknown 8 Byte block of data
        /// </summary>
        [Size(8)]
        public byte[] Unknown;
        /// <summary>
        /// Specifies Identifier used to identify Tag being referenced
        /// </summary>
        public short Ident;
        /// <summary>
        /// Specifies the Index in the Tag table of the tag being referenced
        /// </summary>
        public short Index;
        #endregion
        /// <summary>
        /// What type of ID is this
        /// </summary>
        private IdentType IDType;
        /// <summary>
        /// Creates a new instance of TagReference
        /// </summary>
        public TagReference() { }
        /// <summary>
        /// Creates a new instance of TagReference and reads chunk structure
        /// </summary>
        /// <param name="br">Stream to read from</param>
        public TagReference(EndianReader br, IdentType type)
        {
            IDType = type;
            Read(br);
        }
        /// <summary>
        /// Reads TagReference Chunk structure from stream
        /// </summary>
        /// <param name="br">Stream to read from</param>
        public override void Read(EndianReader br)
        {
            //If Ident is a tag reference
            if (IDType == IdentType.TagReference)
            {
                //Read Tag Class as char array and create a new string
                TagClass = new string(br.ReadChars(4));
                //Read Unknown Data
                Unknown = br.ReadBytes(8);
            }
            //Read Ident
            Ident = br.ReadInt16();
            //Read Index
            Index = br.ReadInt16();
        }
        /// <summary>
        /// Writes TagReference Chunk structure to stream
        /// </summary>
        /// <param name="bw">Stream to write to</param>
        public override void Write(EndianWriter bw)
        {
            //If Ident is a tag reference
            if (IDType == IdentType.TagReference)
            {
                //Write Tag Class as a Char Array
                bw.Write(TagClass.ToCharArray());
                //Write Unknown Data
                bw.Write(Unknown);
            }
            //Write Ident
            bw.Write(Ident);
            //Write Index
            bw.Write(Index);
        }
    }
}