﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Sandbox.Usermap;
using Sandbox.Forms;
using Sandbox.Controls;
using Sandbox.Editor;
using System.IO;
using System.Runtime.Serialization.Formatters.Binary;
using System.Xml.Serialization;
using Sandbox.Plugins;

namespace Sandbox
{
    public partial class Form1 : Form
    {
        /// <summary>
        /// Current version of application.
        /// </summary>
        public static double CurrentVersion = 3.0;
        /// <summary>
        /// Current release type.
        /// </summary>
        public static BuildType CurrentBuildType = BuildType.Stable;

        /// <summary>
        /// Defines release types of the application.
        /// </summary>
        public enum BuildType
        {
            Internal_Release,
            Stable,
            Unstable,
            Beta,
        }

        public Form1()
        {
            InitializeComponent();
            //AppSettings.settingsDirectory = Application.StartupPath;
            //AppSettings.LoadSettings();
            LoadImages();                   
        }
        /// <summary>
        /// Loads images from the image folder into 
        ///  the images lists in the global variables class
        /// </summary>
        public void LoadImages()
        {
            GlobalVariables.ImageIcons = new ImageList();
            GlobalVariables.ImageIcons.ImageSize = new Size(64, 64);
            GlobalVariables.SmallImageIcons = new ImageList();
            GlobalVariables.SmallImageIcons.ImageSize = new Size(32, 32);
            DirectoryInfo di = new DirectoryInfo(Application.StartupPath + "\\Images");
            FileInfo[] fi = di.GetFiles();
            for(int i = 0; i < fi.Length; i++)
            {
                Image im = Image.FromFile(fi[i].FullName);
                GlobalVariables.ImageIcons.Images.Add(im);
                GlobalVariables.ImageIcons.Images.SetKeyName(i, fi[i].Name.Replace(".png", ""));
                GlobalVariables.SmallImageIcons.Images.Add(im);
                GlobalVariables.SmallImageIcons.Images.SetKeyName(i, fi[i].Name.Replace(".png", ""));
            }
        }

        private void openMToolStripMenuItem_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Title = "Select a Halo 3 usermap...";
            ofd.Filter = "Map Varients (.map)|*.map";
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                GlobalVariables.Usermap = new H3Usermap(ofd.FileName);

                int error = GlobalVariables.Usermap.Read();

                if (error == 0)
                {

                }
                else if (error == 1)
                {
                    MessageBox.Show("You must extract the usermap from the con file before opening.");
                    return;
                }
                else if (error == 2)
                {
                    MessageBox.Show("File was not a usermap.", "Error incorrect format.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;

                }
                else if (error == 3)
                {
                    MessageBox.Show("An error was encountered reading this usermap.", "Error reading file.", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                string path = Misc.GetPluginName((int)GlobalVariables.Usermap.header.MapId);
                if (File.Exists(path))
                {
                    GlobalVariables.Plugin = new MapPlugin(path);
                }
                else
                {
                    MessageBox.Show("Could not find a plugin for this usermap.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                //Load items into listview
                listpanel1.Populate();
            }
        }

        private void settingsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            PropertyEditor pe = new PropertyEditor();
            pe.SelectedObject = AppSettings.Settings;
            pe.FormClosing += new FormClosingEventHandler(settings_FormClosing);
            pe.Text = "Settings";
            pe.ShowDialog();
        }

        void settings_FormClosing(object sender, FormClosingEventArgs e)
        {
            AppSettings.SaveSettings();
        }

        private void aboutToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            //Create new about menu dialog
            About ab = new About();
            //Show about menu dialog
            ab.ShowDialog();
        } 

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Close application and all open forms
            Application.Exit();
        }

        private void closeUsermapToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //Close usermap file io
            GlobalVariables.Usermap.Close();
            //Clear listview items
            listpanel1.Clear();
        }
    }
    /// <summary>
    /// Holds variables that need to be accessable from anywhere.
    /// </summary>
    public static class GlobalVariables
    {
        private static MapPlugin plugin;
        private static H3Usermap umap;
        private static object clipboard;        
        private static ImageList imageIcons;
        private static ImageList smallImageIcons;
        /// <summary>
        /// Gives info about the map file for the currently open usermap
        /// </summary>
        public static MapPlugin Plugin
        {
            get { return plugin; }
            set { plugin = value; }
        }
        /// <summary>
        /// Reads, Writes, and Holds information for the usermap being edited.
        /// </summary>
        public static H3Usermap Usermap
        {
            get { return umap; }
            set { umap = value; }
        }
        /// <summary>
        /// Holds a placement block for copy/paste functions.
        /// </summary>
        public static object Clipboard
        {
            get { return clipboard; }
            set { clipboard = value; }
        }
        /// <summary>
        /// Lists all the large Icons for use with the listview
        /// </summary>
        public static ImageList ImageIcons
        {
            get { return imageIcons; }
            set { imageIcons = value; }
        }
        /// <summary>
        /// Lists all the small Icons for use with the listview
        /// </summary>
        public static ImageList SmallImageIcons
        {
            get { return smallImageIcons; }
            set { smallImageIcons = value; }
        }
    }
}
